@@ -1,4 +1,4 @@
-/* $Id: newgrf.h 23887 2012-02-04 13:29:04Z michi_cc $ */
+/* $Id: newgrf.h 27730 2017-01-14 15:48:19Z frosch $ */
 
 /*
  * This file is part of OpenTTD.
@@ -16,6 +16,8 @@
 #include "rail_type.h"
 #include "fileio_type.h"
 #include "core/bitmath_func.hpp"
+#include "core/alloc_type.hpp"
+#include "core/smallvec_type.hpp"
 
 /**
  * List of different canal 'features'.
@@ -59,6 +61,7 @@ enum GrfMiscBit {
 	GMB_TRAIN_WIDTH_32_PIXELS  = 3, ///< Use 32 pixels per train vehicle in depot gui and vehicle details. Never set in the global variable; @see GRFFile::traininfo_vehicle_width
 	GMB_AMBIENT_SOUND_CALLBACK = 4,
 	GMB_CATENARY_ON_3RD_TRACK  = 5, // Unsupported.
+	GMB_SECOND_ROCKY_TILE_SET  = 6,
 };
 
 enum GrfSpecFeature {
@@ -98,9 +101,8 @@ struct GRFLabel {
 };
 
 /** Dynamic data of a loaded NewGRF */
-struct GRFFile {
+struct GRFFile : ZeroedMemoryAllocator {
 	char *filename;
-	bool is_ottdfile;
 	uint32 grfid;
 	byte grf_version;
 
@@ -120,13 +122,11 @@ struct GRFFile {
 
 	GRFLabel *label; ///< Pointer to the first label. This is a linked list, not an array.
 
-	uint8 cargo_max;
-	CargoLabel *cargo_list;
-	uint8 cargo_map[NUM_CARGO];
+	SmallVector<CargoLabel, 4> cargo_list;          ///< Cargo translation table (local ID -> label)
+	uint8 cargo_map[NUM_CARGO];                     ///< Inverse cargo translation table (CargoID -> local ID)
 
-	uint8 railtype_max;
-	RailTypeLabel *railtype_list;
-	RailType railtype_map[RAILTYPE_END];
+	SmallVector<RailTypeLabel, 4> railtype_list;    ///< Railtype translation table
+	RailTypeByte railtype_map[RAILTYPE_END];
 
 	CanalProperties canal_local_properties[CF_END]; ///< Canal properties as set by this NewGRF
 
@@ -138,6 +138,9 @@ struct GRFFile {
 	uint32 grf_features;                     ///< Bitset of GrfSpecFeature the grf uses
 	PriceMultipliers price_base_multipliers; ///< Price base multipliers as set by the grf.
 
+	GRFFile(const struct GRFConfig *config);
+	~GRFFile();
+
 	/** Get GRF Parameter with range checking */
 	uint32 GetParam(uint number) const
 	{
@@ -164,7 +167,7 @@ struct GRFLoadedFeatures {
 };
 
 /**
- * Check for grf miscelaneous bits
+ * Check for grf miscellaneous bits
  * @param bit The bit to check.
  * @return Whether the bit is set.
  */
@@ -180,7 +183,7 @@ extern GRFLoadedFeatures _loaded_newgrf_features;
 byte GetGRFContainerVersion();
 
 void LoadNewGRFFile(struct GRFConfig *config, uint file_index, GrfLoadingStage stage, Subdirectory subdir);
-void LoadNewGRF(uint load_index, uint file_index);
+void LoadNewGRF(uint load_index, uint file_index, uint num_baseset);
 void ReloadNewGRFData(); // in saveload/afterload.cpp
 void ResetNewGRFData();
 void ResetPersistentNewGRFData();
