@@ -1,4 +1,4 @@
-/* $Id: newgrf_engine.cpp 24604 2012-10-17 18:59:28Z rubidium $ */
+/* $Id: newgrf_engine.cpp 27984 2018-03-11 13:19:41Z frosch $ */
 
 /*
  * This file is part of OpenTTD.
@@ -25,6 +25,8 @@
 #include "newgrf_railtype.h"
 #include "ship.h"
 
+#include "safeguards.h"
+
 struct WagonOverride {
 	EngineID *train_id;
 	uint trains;
@@ -54,11 +56,6 @@ const SpriteGroup *GetWagonOverrideSpriteSet(EngineID engine, CargoID cargo, Eng
 {
 	const Engine *e = Engine::Get(engine);
 
-	/* XXX: This could turn out to be a timesink on profiles. We could
-	 * always just dedicate 65535 bytes for an [engine][train] trampoline
-	 * for O(1). Or O(logMlogN) and searching binary tree or smt. like
-	 * that. --pasky */
-
 	for (uint i = 0; i < e->overrides_count; i++) {
 		const WagonOverride *wo = &e->overrides[i];
 
@@ -177,22 +174,22 @@ static byte MapAircraftMovementState(const Aircraft *v)
 	switch (v->state) {
 		case HANGAR:
 			/* The international airport is a special case as helicopters can land in
-			 * front of the hanger. Helicopters also change their air.state to
+			 * front of the hangar. Helicopters also change their air.state to
 			 * AMED_HELI_LOWER some time before actually descending. */
 
 			/* This condition only occurs for helicopters, during descent,
-			 * to a landing by the hanger of an international airport. */
+			 * to a landing by the hangar of an international airport. */
 			if (amdflag & AMED_HELI_LOWER) return AMS_TTDP_HELI_LAND_AIRPORT;
 
 			/* This condition only occurs for helicopters, before starting descent,
-			 * to a landing by the hanger of an international airport. */
+			 * to a landing by the hangar of an international airport. */
 			if (amdflag & AMED_SLOWTURN) return AMS_TTDP_FLIGHT_TO_TOWER;
 
 			/* The final two conditions apply to helicopters or aircraft.
-			 * Has reached hanger? */
+			 * Has reached hangar? */
 			if (amdflag & AMED_EXACTPOS) return AMS_TTDP_HANGAR;
 
-			/* Still moving towards hanger. */
+			/* Still moving towards hangar. */
 			return AMS_TTDP_TO_HANGAR;
 
 		case TERM1:
@@ -320,7 +317,7 @@ static byte MapAircraftMovementAction(const Aircraft *v)
 		case STARTTAKEOFF: // Accelerating down runway
 		case ENDTAKEOFF:   // Ascent
 		case HELITAKEOFF:
-			/* @todo Need to find which terminal (or hanger) we've come from. How? */
+			/* @todo Need to find which terminal (or hangar) we've come from. How? */
 			return AMA_TTDP_PAD1_TO_TAKEOFF;
 
 		case FLYING:
@@ -340,79 +337,63 @@ static byte MapAircraftMovementAction(const Aircraft *v)
 }
 
 
-/* Vehicle Resolver Functions */
-static inline const Vehicle *GRV(const ResolverObject *object)
+/* virtual */ uint32 VehicleScopeResolver::GetRandomBits() const
 {
-	switch (object->scope) {
-		default: NOT_REACHED();
-		case VSG_SCOPE_SELF: return object->u.vehicle.self;
-		case VSG_SCOPE_PARENT: return object->u.vehicle.parent;
+	return this->v == NULL ? 0 : this->v->random_bits;
+}
+
+/* virtual */ uint32 VehicleScopeResolver::GetTriggers() const
+{
+	return this->v == NULL ? 0 : this->v->waiting_triggers;
+}
+
+
+/* virtual */ ScopeResolver *VehicleResolverObject::GetScope(VarSpriteGroupScope scope, byte relative)
+{
+	switch (scope) {
+		case VSG_SCOPE_SELF:   return &this->self_scope;
+		case VSG_SCOPE_PARENT: return &this->parent_scope;
 		case VSG_SCOPE_RELATIVE: {
-			if (object->u.vehicle.self == NULL) return NULL;
+			int32 count = GB(relative, 0, 4);
+			if (this->self_scope.v != NULL && (relative != this->cached_relative_count || count == 0)) {
+				/* Note: This caching only works as long as the VSG_SCOPE_RELATIVE cannot be used in
+				 *       VarAct2 with procedure calls. */
+				if (count == 0) count = GetRegister(0x100);
 
-			int32 count = GB(object->count, 0, 4);
-			if (count == 0) count = GetRegister(0x100);
-
-			const Vehicle *v = NULL;
-			switch (GB(object->count, 6, 2)) {
-				default: NOT_REACHED();
-				case 0x00: // count back (away from the engine), starting at this vehicle
-					v = object->u.vehicle.self;
-					break;
-				case 0x01: // count forward (toward the engine), starting at this vehicle
-					v = object->u.vehicle.self;
-					count = -count;
-					break;
-				case 0x02: // count back, starting at the engine
-					v = object->u.vehicle.parent;
-					break;
-				case 0x03: { // count back, starting at the first vehicle in this chain of vehicles with the same ID, as for vehicle variable 41
-					const Vehicle *self = object->u.vehicle.self;
-					for (const Vehicle *u = self->First(); u != self; u = u->Next()) {
-						if (u->engine_type != self->engine_type) {
-							v = NULL;
-						} else {
-							if (v == NULL) v = u;
+				const Vehicle *v = NULL;
+				switch (GB(relative, 6, 2)) {
+					default: NOT_REACHED();
+					case 0x00: // count back (away from the engine), starting at this vehicle
+						v = this->self_scope.v;
+						break;
+					case 0x01: // count forward (toward the engine), starting at this vehicle
+						v = this->self_scope.v;
+						count = -count;
+						break;
+					case 0x02: // count back, starting at the engine
+						v = this->parent_scope.v;
+						break;
+					case 0x03: { // count back, starting at the first vehicle in this chain of vehicles with the same ID, as for vehicle variable 41
+						const Vehicle *self = this->self_scope.v;
+						for (const Vehicle *u = self->First(); u != self; u = u->Next()) {
+							if (u->engine_type != self->engine_type) {
+								v = NULL;
+							} else {
+								if (v == NULL) v = u;
+							}
 						}
+						if (v == NULL) v = self;
+						break;
 					}
-					if (v == NULL) v = self;
-					break;
 				}
+				this->relative_scope.SetVehicle(v->Move(count));
 			}
-			return v->Move(count);
+			return &this->relative_scope;
 		}
+		default: return ResolverObject::GetScope(scope, relative);
 	}
 }
 
-
-static uint32 VehicleGetRandomBits(const ResolverObject *object)
-{
-	return GRV(object) == NULL ? 0 : GRV(object)->random_bits;
-}
-
-
-static uint32 VehicleGetTriggers(const ResolverObject *object)
-{
-	return GRV(object) == NULL ? 0 : GRV(object)->waiting_triggers;
-}
-
-
-static void VehicleSetTriggers(const ResolverObject *object, int triggers)
-{
-	/* Evil cast to get around const-ness. This used to be achieved by an
-	 * innocent looking function pointer cast... Currently I cannot see a
-	 * way of avoiding this without removing consts deep within gui code.
-	 */
-	Vehicle *v = const_cast<Vehicle *>(GRV(object));
-
-	/* This function must only be called when processing triggers -- any
-	 * other time is an error. */
-	assert(object->trigger != 0);
-
-	if (v != NULL) v->waiting_triggers = triggers;
-}
-
-
 /**
  * Determines the livery of an engine.
  *
@@ -464,7 +445,7 @@ static uint32 PositionHelper(const Vehicle *v, bool consecutive)
 	return chain_before | chain_after << 8 | (chain_before + chain_after + consecutive) << 16;
 }
 
-static uint32 VehicleGetVariable(Vehicle *v, const ResolverObject *object, byte variable, uint32 parameter, bool *available)
+static uint32 VehicleGetVariable(Vehicle *v, const VehicleScopeResolver *object, byte variable, uint32 parameter, bool *available)
 {
 	/* Calculated vehicle parameters */
 	switch (variable) {
@@ -544,10 +525,15 @@ static uint32 VehicleGetVariable(Vehicle *v, const ResolverObject *object, byte
 			/* The cargo translation is specific to the accessing GRF, and thus cannot be cached. */
 			CargoID common_cargo_type = (v->grf_cache.consist_cargo_information >> 8) & 0xFF;
 
-			/* Unlike everywhere else the cargo translation table is only used since grf version 8, not 7.
-			 * Note: The grffile == NULL case only happens if this function is called for default vehicles.
-			 *       And this is only done by CheckCaches(). */
-			const GRFFile *grffile = object->grffile;
+			/* Note:
+			 *  - Unlike everywhere else the cargo translation table is only used since grf version 8, not 7.
+			 *  - For translating the cargo type we need to use the GRF which is resolving the variable, which
+			 *    is object->ro.grffile.
+			 *    In case of CBID_TRAIN_ALLOW_WAGON_ATTACH this is not the same as v->GetGRF().
+			 *  - The grffile == NULL case only happens if this function is called for default vehicles.
+			 *    And this is only done by CheckCaches().
+			 */
+			const GRFFile *grffile = object->ro.grffile;
 			uint8 common_bitnum = (common_cargo_type == CT_INVALID) ? 0xFF :
 				(grffile == NULL || grffile->grf_version < 8) ? CargoSpec::Get(common_cargo_type)->bitnum : grffile->cargo_map[common_cargo_type];
 
@@ -566,7 +552,7 @@ static uint32 VehicleGetVariable(Vehicle *v, const ResolverObject *object, byte
 
 			{
 				const Vehicle *w = v->Next();
-				uint16 altitude = v->z_pos - w->z_pos; // Aircraft height - shadow height
+				uint16 altitude = ClampToU16(v->z_pos - w->z_pos); // Aircraft height - shadow height
 				byte airporttype = ATP_TTDP_LARGE;
 
 				const Station *st = GetTargetAirportIfValid(Aircraft::From(v));
@@ -609,7 +595,12 @@ static uint32 VehicleGetVariable(Vehicle *v, const ResolverObject *object, byte
 			 */
 			const CargoSpec *cs = CargoSpec::Get(v->cargo_type);
 
-			return (cs->classes << 16) | (cs->weight << 8) | v->GetGRF()->cargo_map[v->cargo_type];
+			/* Note:
+			 * For translating the cargo type we need to use the GRF which is resolving the variable, which
+			 * is object->ro.grffile.
+			 * In case of CBID_TRAIN_ALLOW_WAGON_ATTACH this is not the same as v->GetGRF().
+			 */
+			return (cs->classes << 16) | (cs->weight << 8) | object->ro.grffile->cargo_map[v->cargo_type];
 		}
 
 		case 0x48: return v->GetEngine()->flags; // Vehicle Type Info
@@ -618,14 +609,29 @@ static uint32 VehicleGetVariable(Vehicle *v, const ResolverObject *object, byte
 		case 0x4A: {
 			if (v->type != VEH_TRAIN) return 0;
 			RailType rt = GetTileRailType(v->tile);
-			return (HasPowerOnRail(Train::From(v)->railtype, rt) ? 0x100 : 0) | GetReverseRailTypeTranslation(rt, object->grffile);
+			return (HasPowerOnRail(Train::From(v)->railtype, rt) ? 0x100 : 0) | GetReverseRailTypeTranslation(rt, object->ro.grffile);
 		}
 
 		case 0x4B: // Long date of last service
 			return v->date_of_last_service;
 
+		case 0x4C: // Current maximum speed in NewGRF units
+			if (!v->IsPrimaryVehicle()) return 0;
+			return v->GetCurrentMaxSpeed();
+
+		case 0x4D: // Position within articulated vehicle
+			if (!HasBit(v->grf_cache.cache_valid, NCVV_POSITION_IN_VEHICLE)) {
+				byte artic_before = 0;
+				for (const Vehicle *u = v; u->IsArticulatedPart(); u = u->Previous()) artic_before++;
+				byte artic_after = 0;
+				for (const Vehicle *u = v; u->HasArticulatedPart(); u = u->Next()) artic_after++;
+				v->grf_cache.position_in_vehicle = artic_before | artic_after << 8;
+				SetBit(v->grf_cache.cache_valid, NCVV_POSITION_IN_VEHICLE);
+			}
+			return v->grf_cache.position_in_vehicle;
+
 		/* Variables which use the parameter */
-		case 0x60: // Count consist's engine ID occurance
+		case 0x60: // Count consist's engine ID occurrence
 			if (v->type != VEH_TRAIN) return v->GetEngine()->grf_prop.local_id == parameter ? 1 : 0;
 
 			{
@@ -637,16 +643,27 @@ static uint32 VehicleGetVariable(Vehicle *v, const ResolverObject *object, byte
 			}
 
 		case 0x61: // Get variable of n-th vehicle in chain [signed number relative to vehicle]
-			if (!v->IsGroundVehicle() || parameter == 0x61) return 0;
+			if (!v->IsGroundVehicle() || parameter == 0x61) {
+				/* Not available */
+				break;
+			}
 
 			/* Only allow callbacks that don't change properties to avoid circular dependencies. */
-			if (object->callback == CBID_NO_CALLBACK || object->callback == CBID_TRAIN_ALLOW_WAGON_ATTACH || object->callback == CBID_VEHICLE_START_STOP_CHECK || object->callback == CBID_VEHICLE_32DAY_CALLBACK) {
+			if (object->ro.callback == CBID_NO_CALLBACK || object->ro.callback == CBID_RANDOM_TRIGGER || object->ro.callback == CBID_TRAIN_ALLOW_WAGON_ATTACH ||
+					object->ro.callback == CBID_VEHICLE_START_STOP_CHECK || object->ro.callback == CBID_VEHICLE_32DAY_CALLBACK || object->ro.callback == CBID_VEHICLE_COLOUR_MAPPING ||
+					object->ro.callback == CBID_VEHICLE_SPAWN_VISUAL_EFFECT) {
 				Vehicle *u = v->Move((int32)GetRegister(0x10F));
-				if (u == NULL) return 0;
+				if (u == NULL) return 0; // available, but zero
 
-				return VehicleGetVariable(u, object, parameter, GetRegister(0x10E), available);
+				if (parameter == 0x5F) {
+					/* This seems to be the only variable that makes sense to access via var 61, but is not handled by VehicleGetVariable */
+					return (u->random_bits << 8) | u->waiting_triggers;
+				} else {
+					return VehicleGetVariable(u, object, parameter, GetRegister(0x10E), available);
+				}
 			}
-			return 0;
+			/* Not available */
+			break;
 
 		case 0x62: { // Curvature/position difference for n-th vehicle in chain [signed number relative to vehicle]
 			/* Format: zzyyxxFD
@@ -692,6 +709,7 @@ static uint32 VehicleGetVariable(Vehicle *v, const ResolverObject *object, byte
 				if (powered && !has_power) SetBit(modflags, 6);
 				if (HasBit(t->flags, VRF_TOGGLE_REVERSE)) SetBit(modflags, 8);
 			}
+			if (HasBit(v->vehicle_flags, VF_CARGO_UNLOADING)) SetBit(modflags, 1);
 			if (HasBit(v->vehicle_flags, VF_BUILT_AS_PROTOTYPE)) SetBit(modflags, 10);
 
 			return variable == 0xFE ? modflags : GB(modflags, 8, 8);
@@ -724,8 +742,8 @@ static uint32 VehicleGetVariable(Vehicle *v, const ResolverObject *object, byte
 		}
 		case 0x12: return Clamp(v->date_of_last_service - DAYS_TILL_ORIGINAL_BASE_YEAR, 0, 0xFFFF);
 		case 0x13: return GB(Clamp(v->date_of_last_service - DAYS_TILL_ORIGINAL_BASE_YEAR, 0, 0xFFFF), 8, 8);
-		case 0x14: return v->service_interval;
-		case 0x15: return GB(v->service_interval, 8, 8);
+		case 0x14: return v->GetServiceInterval();
+		case 0x15: return GB(v->GetServiceInterval(), 8, 8);
 		case 0x16: return v->last_station_visited;
 		case 0x17: return v->tick_counter;
 		case 0x18:
@@ -747,7 +765,7 @@ static uint32 VehicleGetVariable(Vehicle *v, const ResolverObject *object, byte
 		case 0x1C: return v->y_pos;
 		case 0x1D: return GB(v->y_pos, 8, 8);
 		case 0x1E: return v->z_pos;
-		case 0x1F: return object->u.vehicle.info_view ? DIR_W : v->direction;
+		case 0x1F: return object->info_view ? DIR_W : v->direction;
 		case 0x28: return 0; // cur_image is a potential desyncer due to Action1 in static NewGRFs.
 		case 0x29: return 0; // cur_image is a potential desyncer due to Action1 in static NewGRFs.
 		case 0x32: return v->vehstatus;
@@ -759,8 +777,8 @@ static uint32 VehicleGetVariable(Vehicle *v, const ResolverObject *object, byte
 		case 0x39: return v->cargo_type;
 		case 0x3A: return v->cargo_cap;
 		case 0x3B: return GB(v->cargo_cap, 8, 8);
-		case 0x3C: return ClampToU16(v->cargo.Count());
-		case 0x3D: return GB(ClampToU16(v->cargo.Count()), 8, 8);
+		case 0x3C: return ClampToU16(v->cargo.StoredCount());
+		case 0x3D: return GB(ClampToU16(v->cargo.StoredCount()), 8, 8);
 		case 0x3E: return v->cargo.Source();
 		case 0x3F: return ClampU(v->cargo.DaysInTransit(), 0, 0xFF);
 		case 0x40: return ClampToU16(v->age);
@@ -862,26 +880,24 @@ static uint32 VehicleGetVariable(Vehicle *v, const ResolverObject *object, byte
 	return UINT_MAX;
 }
 
-static uint32 VehicleGetVariable(const ResolverObject *object, byte variable, uint32 parameter, bool *available)
+/* virtual */ uint32 VehicleScopeResolver::GetVariable(byte variable, uint32 parameter, bool *available) const
 {
-	Vehicle *v = const_cast<Vehicle*>(GRV(object));
-
-	if (v == NULL) {
+	if (this->v == NULL) {
 		/* Vehicle does not exist, so we're in a purchase list */
 		switch (variable) {
-			case 0x43: return GetCompanyInfo(_current_company, LiveryHelper(object->u.vehicle.self_type, NULL)); // Owner information
+			case 0x43: return GetCompanyInfo(_current_company, LiveryHelper(this->self_type, NULL)); // Owner information
 			case 0x46: return 0;               // Motion counter
 			case 0x47: { // Vehicle cargo info
-				const Engine *e = Engine::Get(object->u.vehicle.self_type);
+				const Engine *e = Engine::Get(this->self_type);
 				CargoID cargo_type = e->GetDefaultCargoType();
 				if (cargo_type != CT_INVALID) {
 					const CargoSpec *cs = CargoSpec::Get(cargo_type);
-					return (cs->classes << 16) | (cs->weight << 8) | e->GetGRF()->cargo_map[cargo_type];
+					return (cs->classes << 16) | (cs->weight << 8) | this->ro.grffile->cargo_map[cargo_type];
 				} else {
 					return 0x000000FF;
 				}
 			}
-			case 0x48: return Engine::Get(object->u.vehicle.self_type)->flags; // Vehicle Type Info
+			case 0x48: return Engine::Get(this->self_type)->flags; // Vehicle Type Info
 			case 0x49: return _cur_year; // 'Long' format build year
 			case 0x4B: return _date; // Long date of last service
 			case 0x92: return Clamp(_date - DAYS_TILL_ORIGINAL_BASE_YEAR, 0, 0xFFFF); // Date of last service
@@ -895,13 +911,13 @@ static uint32 VehicleGetVariable(const ResolverObject *object, byte variable, ui
 		return UINT_MAX;
 	}
 
-	return VehicleGetVariable(v, object, variable, parameter, available);
+	return VehicleGetVariable(const_cast<Vehicle*>(this->v), this, variable, parameter, available);
 }
 
 
-static const SpriteGroup *VehicleResolveReal(const ResolverObject *object, const RealSpriteGroup *group)
+/* virtual */ const SpriteGroup *VehicleResolverObject::ResolveReal(const RealSpriteGroup *group) const
 {
-	const Vehicle *v = object->u.vehicle.self;
+	const Vehicle *v = this->self_scope.v;
 
 	if (v == NULL) {
 		if (group->num_loading > 0) return group->loading[0];
@@ -915,97 +931,91 @@ static const SpriteGroup *VehicleResolveReal(const ResolverObject *object, const
 
 	if (totalsets == 0) return NULL;
 
-	uint set = (v->cargo.Count() * totalsets) / max((uint16)1, v->cargo_cap);
+	uint set = (v->cargo.StoredCount() * totalsets) / max((uint16)1, v->cargo_cap);
 	set = min(set, totalsets - 1);
 
 	return in_motion ? group->loaded[set] : group->loading[set];
 }
 
-
-static inline void NewVehicleResolver(ResolverObject *res, EngineID engine_type, const Vehicle *v)
+/**
+ * Get the grf file associated with an engine type.
+ * @param engine_type Engine to query.
+ * @return grf file associated with the engine.
+ */
+static const GRFFile *GetEngineGrfFile(EngineID engine_type)
 {
-	res->GetRandomBits = &VehicleGetRandomBits;
-	res->GetTriggers   = &VehicleGetTriggers;
-	res->SetTriggers   = &VehicleSetTriggers;
-	res->GetVariable   = &VehicleGetVariable;
-	res->ResolveReal   = &VehicleResolveReal;
-
-	res->u.vehicle.self   = v;
-	res->u.vehicle.parent = (v != NULL) ? v->First() : v;
-
-	res->u.vehicle.self_type = engine_type;
-	res->u.vehicle.info_view = false;
-
-	res->callback        = CBID_NO_CALLBACK;
-	res->callback_param1 = 0;
-	res->callback_param2 = 0;
-	res->ResetState();
-
 	const Engine *e = Engine::Get(engine_type);
-	res->grffile         = (e != NULL ? e->GetGRF() : NULL);
+	return (e != NULL) ? e->GetGRF() : NULL;
 }
 
-
 /**
- * Retrieve the SpriteGroup for the specified vehicle.
- * If the vehicle is not specified, the purchase list group for the engine is
- * chosen. For trains, an additional engine override lookup is performed.
- * @param engine    Engine type of the vehicle.
- * @param v         The vehicle itself.
- * @param use_cache Use cached override
- * @returns         The selected SpriteGroup for the vehicle.
+ * Resolver of a vehicle (chain).
+ * @param engine_type Engine type
+ * @param v %Vehicle being resolved.
+ * @param wagon_override Application of wagon overrides.
+ * @param info_view Indicates if the item is being drawn in an info window.
+ * @param callback Callback ID.
+ * @param callback_param1 First parameter (var 10) of the callback.
+ * @param callback_param2 Second parameter (var 18) of the callback.
  */
-static const SpriteGroup *GetVehicleSpriteGroup(EngineID engine, const Vehicle *v, bool use_cache = true)
+VehicleResolverObject::VehicleResolverObject(EngineID engine_type, const Vehicle *v, WagonOverride wagon_override, bool info_view,
+		CallbackID callback, uint32 callback_param1, uint32 callback_param2)
+	: ResolverObject(GetEngineGrfFile(engine_type), callback, callback_param1, callback_param2),
+	self_scope(*this, engine_type, v, info_view),
+	parent_scope(*this, engine_type, ((v != NULL) ? v->First() : v), info_view),
+	relative_scope(*this, engine_type, v, info_view),
+	cached_relative_count(0)
 {
-	const SpriteGroup *group;
-	CargoID cargo;
-
-	if (v == NULL) {
-		cargo = CT_PURCHASE;
+	if (wagon_override == WO_SELF) {
+		this->root_spritegroup = GetWagonOverrideSpriteSet(engine_type, CT_DEFAULT, engine_type);
 	} else {
-		cargo = v->cargo_type;
+		if (wagon_override != WO_NONE && v != NULL && v->IsGroundVehicle()) {
+			assert(v->engine_type == engine_type); // overrides make little sense with fake scopes
 
-		if (v->IsGroundVehicle()) {
 			/* For trains we always use cached value, except for callbacks because the override spriteset
 			 * to use may be different than the one cached. It happens for callback 0x15 (refit engine),
 			 * as v->cargo_type is temporary changed to the new type */
-			if (use_cache && v->type == VEH_TRAIN) {
-				group = Train::From(v)->tcache.cached_override;
+			if (wagon_override == WO_CACHED && v->type == VEH_TRAIN) {
+				this->root_spritegroup = Train::From(v)->tcache.cached_override;
 			} else {
-				group = GetWagonOverrideSpriteSet(v->engine_type, v->cargo_type, v->GetGroundVehicleCache()->first_engine);
+				this->root_spritegroup = GetWagonOverrideSpriteSet(v->engine_type, v->cargo_type, v->GetGroundVehicleCache()->first_engine);
 			}
-			if (group != NULL) return group;
+		}
+
+		if (this->root_spritegroup == NULL) {
+			const Engine *e = Engine::Get(engine_type);
+			CargoID cargo = v != NULL ? v->cargo_type : CT_PURCHASE;
+			assert(cargo < lengthof(e->grf_prop.spritegroup));
+			this->root_spritegroup = e->grf_prop.spritegroup[cargo] != NULL ? e->grf_prop.spritegroup[cargo] : e->grf_prop.spritegroup[CT_DEFAULT];
 		}
 	}
-
-	const Engine *e = Engine::Get(engine);
-
-	assert(cargo < lengthof(e->grf_prop.spritegroup));
-	group = e->grf_prop.spritegroup[cargo];
-	if (group != NULL) return group;
-
-	/* Fall back to the default set if the selected cargo type is not defined */
-	return e->grf_prop.spritegroup[CT_DEFAULT];
 }
 
 
-SpriteID GetCustomEngineSprite(EngineID engine, const Vehicle *v, Direction direction, EngineImageType image_type)
+
+void GetCustomEngineSprite(EngineID engine, const Vehicle *v, Direction direction, EngineImageType image_type, VehicleSpriteSeq *result)
 {
-	const SpriteGroup *group;
-	ResolverObject object;
+	VehicleResolverObject object(engine, v, VehicleResolverObject::WO_CACHED, false, CBID_NO_CALLBACK);
+	result->Clear();
 
-	NewVehicleResolver(&object, engine, v);
-
-	object.callback_param1 = image_type;
-
-	group = SpriteGroup::Resolve(GetVehicleSpriteGroup(engine, v), &object);
-	if (group == NULL || group->GetNumResults() == 0) return 0;
-
-	return group->GetResult() + (direction % group->GetNumResults());
+	bool sprite_stack = HasBit(EngInfo(engine)->misc_flags, EF_SPRITE_STACK);
+	uint max_stack = sprite_stack ? lengthof(result->seq) : 1;
+	for (uint stack = 0; stack < max_stack; ++stack) {
+		object.ResetState();
+		object.callback_param1 = image_type | (stack << 8);
+		const SpriteGroup *group = object.Resolve();
+		uint32 reg100 = sprite_stack ? GetRegister(0x100) : 0;
+		if (group != NULL && group->GetNumResults() != 0) {
+			result->seq[result->count].sprite = group->GetResult() + (direction % group->GetNumResults());
+			result->seq[result->count].pal    = GB(reg100, 0, 16); // zero means default recolouring
+			result->count++;
+		}
+		if (!HasBit(reg100, 31)) break;
+	}
 }
 
 
-SpriteID GetRotorOverrideSprite(EngineID engine, const Aircraft *v, bool info_view, EngineImageType image_type)
+void GetRotorOverrideSprite(EngineID engine, const struct Aircraft *v, bool info_view, EngineImageType image_type, VehicleSpriteSeq *result)
 {
 	const Engine *e = Engine::Get(engine);
 
@@ -1013,21 +1023,24 @@ SpriteID GetRotorOverrideSprite(EngineID engine, const Aircraft *v, bool info_vi
 	assert(e->type == VEH_AIRCRAFT);
 	assert(!(e->u.air.subtype & AIR_CTOL));
 
-	ResolverObject object;
+	VehicleResolverObject object(engine, v, VehicleResolverObject::WO_SELF, info_view, CBID_NO_CALLBACK);
+	result->Clear();
+	uint rotor_pos = v == NULL || info_view ? 0 : v->Next()->Next()->state;
 
-	NewVehicleResolver(&object, engine, v);
-
-	object.callback_param1 = image_type;
-	object.u.vehicle.info_view = info_view;
-
-	const SpriteGroup *group = GetWagonOverrideSpriteSet(engine, CT_DEFAULT, engine);
-	group = SpriteGroup::Resolve(group, &object);
-
-	if (group == NULL || group->GetNumResults() == 0) return 0;
-
-	if (v == NULL) return group->GetResult();
-
-	return group->GetResult() + (info_view ? 0 : (v->Next()->Next()->state % group->GetNumResults()));
+	bool sprite_stack = HasBit(e->info.misc_flags, EF_SPRITE_STACK);
+	uint max_stack = sprite_stack ? lengthof(result->seq) : 1;
+	for (uint stack = 0; stack < max_stack; ++stack) {
+		object.ResetState();
+		object.callback_param1 = image_type | (stack << 8);
+		const SpriteGroup *group = object.Resolve();
+		uint32 reg100 = sprite_stack ? GetRegister(0x100) : 0;
+		if (group != NULL && group->GetNumResults() != 0) {
+			result->seq[result->count].sprite = group->GetResult() + (rotor_pos % group->GetNumResults());
+			result->seq[result->count].pal    = GB(reg100, 0, 16); // zero means default recolouring
+			result->count++;
+		}
+		if (!HasBit(reg100, 31)) break;
+	}
 }
 
 
@@ -1044,7 +1057,7 @@ bool UsesWagonOverride(const Vehicle *v)
 
 /**
  * Evaluate a newgrf callback for vehicles
- * @param callback The callback to evalute
+ * @param callback The callback to evaluate
  * @param param1   First parameter of the callback
  * @param param2   Second parameter of the callback
  * @param engine   Engine type of the vehicle to evaluate the callback for
@@ -1053,48 +1066,25 @@ bool UsesWagonOverride(const Vehicle *v)
  */
 uint16 GetVehicleCallback(CallbackID callback, uint32 param1, uint32 param2, EngineID engine, const Vehicle *v)
 {
-	const SpriteGroup *group;
-	ResolverObject object;
-
-	NewVehicleResolver(&object, engine, v);
-
-	object.callback        = callback;
-	object.callback_param1 = param1;
-	object.callback_param2 = param2;
-
-	group = SpriteGroup::Resolve(GetVehicleSpriteGroup(engine, v, false), &object);
-	if (group == NULL) return CALLBACK_FAILED;
-
-	return group->GetCallbackResult();
+	VehicleResolverObject object(engine, v, VehicleResolverObject::WO_UNCACHED, false, callback, param1, param2);
+	return object.ResolveCallback();
 }
 
 /**
  * Evaluate a newgrf callback for vehicles with a different vehicle for parent scope.
- * @param callback The callback to evalute
+ * @param callback The callback to evaluate
  * @param param1   First parameter of the callback
  * @param param2   Second parameter of the callback
  * @param engine   Engine type of the vehicle to evaluate the callback for
- * @param v        The vehicle to evaluate the callback for, or NULL if it doesnt exist yet
+ * @param v        The vehicle to evaluate the callback for, or NULL if it doesn't exist yet
  * @param parent   The vehicle to use for parent scope
  * @return The value the callback returned, or CALLBACK_FAILED if it failed
  */
 uint16 GetVehicleCallbackParent(CallbackID callback, uint32 param1, uint32 param2, EngineID engine, const Vehicle *v, const Vehicle *parent)
 {
-	const SpriteGroup *group;
-	ResolverObject object;
-
-	NewVehicleResolver(&object, engine, v);
-
-	object.callback        = callback;
-	object.callback_param1 = param1;
-	object.callback_param2 = param2;
-
-	object.u.vehicle.parent = parent;
-
-	group = SpriteGroup::Resolve(GetVehicleSpriteGroup(engine, v, false), &object);
-	if (group == NULL) return CALLBACK_FAILED;
-
-	return group->GetCallbackResult();
+	VehicleResolverObject object(engine, v, VehicleResolverObject::WO_NONE, false, callback, param1, param2);
+	object.parent_scope.SetVehicle(parent);
+	return object.ResolveCallback();
 }
 
 
@@ -1116,22 +1106,22 @@ uint GetEngineProperty(EngineID engine, PropertyID property, uint orig_value, co
 
 static void DoTriggerVehicle(Vehicle *v, VehicleTrigger trigger, byte base_random_bits, bool first)
 {
-	const SpriteGroup *group;
-	ResolverObject object;
-	byte new_random_bits;
-
 	/* We can't trigger a non-existent vehicle... */
 	assert(v != NULL);
 
-	NewVehicleResolver(&object, v->engine_type, v);
-	object.callback = CBID_RANDOM_TRIGGER;
-	object.trigger = trigger;
+	VehicleResolverObject object(v->engine_type, v, VehicleResolverObject::WO_CACHED, false, CBID_RANDOM_TRIGGER);
+	object.waiting_triggers = v->waiting_triggers | trigger;
+	v->waiting_triggers = object.waiting_triggers; // store now for var 5F
 
-	group = SpriteGroup::Resolve(GetVehicleSpriteGroup(v->engine_type, v), &object);
+	const SpriteGroup *group = object.Resolve();
 	if (group == NULL) return;
 
-	new_random_bits = Random();
-	uint32 reseed = object.GetReseedSum(); // The scope only affects triggers, not the reseeding
+	/* Store remaining triggers. */
+	v->waiting_triggers = object.GetRemainingTriggers();
+
+	/* Rerandomise bits. Scopes other than SELF are invalid for rerandomisation. For bug-to-bug-compatibility with TTDP we ignore the scope. */
+	byte new_random_bits = Random();
+	uint32 reseed = object.GetReseedSum();
 	v->random_bits &= ~reseed;
 	v->random_bits |= (first ? new_random_bits : base_random_bits) & reseed;
 
@@ -1245,7 +1235,7 @@ void CommitVehicleListOrderChanges()
 	}
 	QSortT(ordering.Begin(), ordering.Length(), EnginePreSort);
 
-	/* Apply Insertion-Sort opeations */
+	/* Apply Insertion-Sort operations */
 	const ListOrderChange *end = _list_order_changes.End();
 	for (const ListOrderChange *it = _list_order_changes.Begin(); it != end; ++it) {
 		EngineID source = it->engine;
@@ -1285,26 +1275,13 @@ void CommitVehicleListOrderChanges()
 	_list_order_changes.Reset();
 }
 
-/**
- * Resolve an engine's spec and such so we can get a variable.
- * @param ro    The resolver object to fill.
- * @param index The vehicle to get the data from.
- */
-void GetVehicleResolver(ResolverObject *ro, uint index)
-{
-	Vehicle *v = Vehicle::Get(index);
-	NewVehicleResolver(ro, v->engine_type, v);
-}
-
 /**
  * Fill the grf_cache of the given vehicle.
  * @param v The vehicle to fill the cache for.
  */
 void FillNewGRFVehicleCache(const Vehicle *v)
 {
-	ResolverObject ro;
-	memset(&ro, 0, sizeof(ro));
-	GetVehicleResolver(&ro, v->index);
+	VehicleResolverObject ro(v->engine_type, v, VehicleResolverObject::WO_NONE);
 
 	/* These variables we have to check; these are the ones with a cache. */
 	static const int cache_entries[][2] = {
@@ -1312,6 +1289,7 @@ void FillNewGRFVehicleCache(const Vehicle *v)
 		{ 0x41, NCVV_POSITION_SAME_ID_LENGTH },
 		{ 0x42, NCVV_CONSIST_CARGO_INFORMATION },
 		{ 0x43, NCVV_COMPANY_INFORMATION },
+		{ 0x4D, NCVV_POSITION_IN_VEHICLE },
 	};
 	assert_compile(NCVV_END == lengthof(cache_entries));
 
@@ -1320,7 +1298,7 @@ void FillNewGRFVehicleCache(const Vehicle *v)
 		/* Only resolve when the cache isn't valid. */
 		if (HasBit(v->grf_cache.cache_valid, cache_entries[i][1])) continue;
 		bool stub;
-		ro.GetVariable(&ro, cache_entries[i][0], 0, &stub);
+		ro.GetScope(VSG_SCOPE_SELF)->GetVariable(cache_entries[i][0], 0, &stub);
 	}
 
 	/* Make sure really all bits are set. */
