@@ -1,4 +1,4 @@
-/* $Id: newgrf_railtype.cpp 27342 2015-07-26 12:25:37Z frosch $ */
+/* $Id: newgrf_railtype.cpp 27984 2018-03-11 13:19:41Z frosch $ */
 
 /*
  * This file is part of OpenTTD.
@@ -67,18 +67,6 @@
 	return NULL;
 }
 
-/**
- * Constructor of the railtype scope resolvers.
- * @param ro Surrounding resolver.
- * @param tile %Tile containing the track. For track on a bridge this is the southern bridgehead.
- * @param context Are we resolving sprites for the upper halftile, or on a bridge?
- */
-RailTypeScopeResolver::RailTypeScopeResolver(ResolverObject &ro, TileIndex tile, TileContext context) : ScopeResolver(ro)
-{
-	this->tile = tile;
-	this->context = context;
-}
-
 /**
  * Resolver object for rail types.
  * @param rti Railtype. NULL in NewGRF Inspect window.
