@@ -1,4 +1,4 @@
-/* $Id: signal.cpp 22017 2011-02-07 22:29:47Z rubidium $ */
+/* $Id: signal.cpp 27893 2017-08-13 18:38:42Z frosch $ */
 
 /*
  * This file is part of OpenTTD.
@@ -18,6 +18,8 @@
 #include "train.h"
 #include "company_base.h"
 
+#include "safeguards.h"
+
 
 /** these are the maximums used for updating signal blocks */
 static const uint SIG_TBU_SIZE    =  64; ///< number of signals entering to block
@@ -52,7 +54,7 @@ template <typename Tdir, uint items>
 struct SmallSet {
 private:
 	uint n;           // actual number of units
-	bool overflowed;  // did we try to oveflow the set?
+	bool overflowed;  // did we try to overflow the set?
 	const char *name; // name, used for debugging purposes...
 
 	/** Element of set */
@@ -73,7 +75,7 @@ public:
 	}
 
 	/**
-	 * Returns value of 'oveflowed'
+	 * Returns value of 'overflowed'
 	 * @return did we try to overflow the set?
 	 */
 	bool Overflowed()
@@ -131,7 +133,7 @@ public:
 	 * Tries to find given tile and dir in the set
 	 * @param tile tile
 	 * @param dir and dir to find
-	 * @return true iff the tile & dir elemnt was found
+	 * @return true iff the tile & dir element was found
 	 */
 	bool IsIn(TileIndex tile, Tdir dir)
 	{
@@ -201,7 +203,7 @@ static Vehicle *TrainOnTileEnum(Vehicle *v, void *)
  * The new and reverse direction is removed from _globset, because we are sure
  * it doesn't need to be checked again
  * Also, remove reverse direction from _tbdset
- * This is the 'core' part so the graph seaching won't enter any tile twice
+ * This is the 'core' part so the graph searching won't enter any tile twice
  *
  * @param t1 tile we are entering
  * @param d1 direction (tile side) we are entering
@@ -227,7 +229,7 @@ static inline bool CheckAddToTodoSet(TileIndex t1, DiagDirection d1, TileIndex t
  * The new and reverse direction is removed from Global set, because we are sure
  * it doesn't need to be checked again
  * Also, remove reverse direction from Todo set
- * This is the 'core' part so the graph seaching won't enter any tile twice
+ * This is the 'core' part so the graph searching won't enter any tile twice
  *
  * @param t1 tile we are entering
  * @param d1 direction (tile side) we are entering
@@ -294,6 +296,7 @@ static SigFlags ExploreSegment(Owner owner)
 					}
 				}
 
+				assert(IsValidDiagDirection(enterdir));
 				TrackBits tracks = GetTrackBits(tile); // trackbits of tile
 				TrackBits tracks_masked = (TrackBits)(tracks & _enterdir_to_trackbits[enterdir]); // only incidating trackbits
 
@@ -501,7 +504,8 @@ static SigSegState UpdateSignalsInBuffer(Owner owner)
 					_tbdset.Add(tile, INVALID_DIAGDIR); // start from depot inside
 					break;
 				}
-				/* FALL THROUGH */
+				FALLTHROUGH;
+
 			case MP_STATION:
 			case MP_ROAD:
 				if ((TrackStatusToTrackBits(GetTileTrackStatus(tile, TRANSPORT_RAIL, 0)) & _enterdir_to_trackbits[dir]) != TRACK_BIT_NONE) {
@@ -510,7 +514,8 @@ static SigSegState UpdateSignalsInBuffer(Owner owner)
 					_tbdset.Add(tile + TileOffsByDiagDir(dir), ReverseDiagDir(dir));
 					break;
 				}
-				/* FALL THROUGH */
+				FALLTHROUGH;
+
 			default:
 				/* jump to next tile */
 				tile = tile + TileOffsByDiagDir(dir);
