@@ -1,4 +1,4 @@
-/* $Id: elrail_data.h 20286 2010-08-01 19:44:49Z frosch $ */
+/* $Id: elrail_data.h 27972 2018-03-05 21:24:48Z peter1138 $ */
 
 /*
  * This file is part of OpenTTD.
@@ -117,7 +117,7 @@ static const byte PreferredPPPofTrackAtPCP[TRACK_END][DIAGDIR_END] = {
 #define NUM_IGNORE_GROUPS 3
 #define IGNORE_NONE 0xFF
 /**
- * In case we have a staight line, we place pylon only every two tiles,
+ * In case we have a straight line, we place pylon only every two tiles,
  * so there are certain tiles which we ignore. A straight line is found if
  * we have exactly two PPPs.
  */
@@ -195,7 +195,7 @@ static const byte IgnoredPCP[NUM_IGNORE_GROUPS][TLG_END][DIAGDIR_END] = {
 
 #undef NO_IGNORE
 
-/** Which pylons can definately NOT be built */
+/** Which pylons can definitely NOT be built */
 static const byte DisallowedPPPofTrackAtPCP[TRACK_END][DIAGDIR_END] = {
 	{1 << DIR_SW | 1 << DIR_NE, 0,           1 << DIR_SW | 1 << DIR_NE, 0          }, // X
 	{0,           1 << DIR_NW | 1 << DIR_SE, 0,           1 << DIR_NW | 1 << DIR_SE}, // Y
@@ -310,10 +310,10 @@ enum WireSpriteOffset {
 	WSO_X_NE_UP,
 	WSO_Y_NW_DOWN,
 
-	WSO_ENTRANCE_NE,
-	WSO_ENTRANCE_SE,
 	WSO_ENTRANCE_SW,
 	WSO_ENTRANCE_NW,
+	WSO_ENTRANCE_NE,
+	WSO_ENTRANCE_SE,
 };
 
 struct SortableSpriteStruct {
@@ -331,7 +331,7 @@ static const uint ELRAIL_ELEVATION = 10;
 /** Wires that a draw one level higher than the north corner. */
 static const uint ELRAIL_ELEVRAISE = ELRAIL_ELEVATION + TILE_HEIGHT;
 
-static const SortableSpriteStruct CatenarySpriteData[] = {
+static const SortableSpriteStruct RailCatenarySpriteData[] = {
 /* X direction
 	 * Flat tiles:
 		 * Wires */
@@ -392,18 +392,18 @@ static const SortableSpriteStruct CatenarySpriteData[] = {
 	{ WSO_EW_E,         15,  8,  3,  3,  1, ELRAIL_ELEVATION }  //!33: LOWER trackbit wire, pylon on both ends
 };
 
-static const SortableSpriteStruct CatenarySpriteData_Depot[] = {
+static const SortableSpriteStruct RailCatenarySpriteData_Depot[] = {
 	{ WSO_ENTRANCE_NE,   0,  7, 15,  1,  1, ELRAIL_ELEVATION }, //! Wire for NE depot exit
 	{ WSO_ENTRANCE_SE,   7,  0,  1, 15,  1, ELRAIL_ELEVATION }, //! Wire for SE depot exit
 	{ WSO_ENTRANCE_SW,   0,  7, 15,  1,  1, ELRAIL_ELEVATION }, //! Wire for SW depot exit
 	{ WSO_ENTRANCE_NW,   7,  0,  1, 15,  1, ELRAIL_ELEVATION }  //! Wire for NW depot exit
 };
 
-static const SortableSpriteStruct CatenarySpriteData_Tunnel[] = {
-	{ WSO_ENTRANCE_NE,   0,  7, 15,  1,  1, ELRAIL_ELEVATION }, //! Wire for NE tunnel exit
-	{ WSO_ENTRANCE_SE,   7,  0,  1, 15,  1, ELRAIL_ELEVATION }, //! Wire for SE tunnel exit
-	{ WSO_ENTRANCE_SW,   0,  7, 15,  1,  1, ELRAIL_ELEVATION }, //! Wire for SW tunnel exit
-	{ WSO_ENTRANCE_NW,   7,  0,  1, 15,  1, ELRAIL_ELEVATION }  //! Wire for NW tunnel exit
+static const SortableSpriteStruct RailCatenarySpriteData_Tunnel[] = {
+	{ WSO_ENTRANCE_SW,   0,  7, 15,  1,  1, ELRAIL_ELEVATION }, //! Wire for NE tunnel (SW facing exit)
+	{ WSO_ENTRANCE_NW,   7,  0,  1, 15,  1, ELRAIL_ELEVATION }, //! Wire for SE tunnel (NW facing exit)
+	{ WSO_ENTRANCE_NE,   0,  7, 15,  1,  1, ELRAIL_ELEVATION }, //! Wire for SW tunnel (NE facing exit)
+	{ WSO_ENTRANCE_SE,   7,  0,  1, 15,  1, ELRAIL_ELEVATION }  //! Wire for NW tunnel (SE facing exit)
 };
 
 
@@ -419,7 +419,7 @@ static const SortableSpriteStruct CatenarySpriteData_Tunnel[] = {
  * <li>Position of the Pylon relative to the track</li>
  * <li>Position of the Pylon inside the tile</li></ol>
  */
-enum CatenarySprite {
+enum RailCatenarySprite {
 	WIRE_X_FLAT_SW,
 	WIRE_X_FLAT_NE,
 	WIRE_X_FLAT_BOTH,
@@ -471,7 +471,7 @@ enum CatenarySprite {
  * c) the second
  * d) both
  * PCP exists.*/
-static const CatenarySprite Wires[5][TRACK_END][4] = {
+static const RailCatenarySprite Wires[5][TRACK_END][4] = {
 	{ // Tileh == 0
 		{INVALID_CATENARY, WIRE_X_FLAT_NE,   WIRE_X_FLAT_SW,   WIRE_X_FLAT_BOTH},
 		{INVALID_CATENARY, WIRE_Y_FLAT_SE,   WIRE_Y_FLAT_NW,   WIRE_Y_FLAT_BOTH},
