@@ -1,4 +1,4 @@
-/* $Id: tile_type.h 21877 2011-01-21 15:48:52Z smatz $ */
+/* $Id: tile_type.h 27987 2018-03-11 13:25:26Z frosch $ */
 
 /*
  * This file is part of OpenTTD.
@@ -12,16 +12,23 @@
 #ifndef TILE_TYPE_H
 #define TILE_TYPE_H
 
-static const uint TILE_SIZE      = 16;            ///< Tiles are 16x16 "units" in size
-static const uint HALF_TILE_SIZE = TILE_SIZE / 2; ///< Half of tile size, to get centre of a tile.
-static const uint TILE_UNIT_MASK = TILE_SIZE - 1; ///< For masking in/out the inner-tile units.
-static const uint TILE_PIXELS    = 32;            ///< a tile is 32x32 pixels
-static const uint TILE_HEIGHT    =  8;            ///< The standard height-difference between tiles on two levels is 8 (z-diff 8)
+static const uint TILE_SIZE           = 16;                    ///< Tile size in world coordinates.
+static const uint TILE_UNIT_MASK      = TILE_SIZE - 1;         ///< For masking in/out the inner-tile world coordinate units.
+static const uint TILE_PIXELS         = 32;                    ///< Pixel distance between tile columns/rows in #ZOOM_LVL_BASE.
+static const uint TILE_HEIGHT         =  8;                    ///< Height of a height level in world coordinate AND in pixels in #ZOOM_LVL_BASE.
 
-static const uint MAX_TILE_HEIGHT     = 15;                    ///< Maximum allowed tile height
+static const uint MAX_BUILDING_PIXELS = 200;                   ///< Maximum height of a building in pixels in #ZOOM_LVL_BASE. (Also applies to "bridge buildings" on the bridge floor.)
+static const int MAX_VEHICLE_PIXEL_X  = 192;                   ///< Maximum width of a vehicle in pixels in #ZOOM_LVL_BASE.
+static const int MAX_VEHICLE_PIXEL_Y  = 96;                    ///< Maximum height of a vehicle in pixels in #ZOOM_LVL_BASE.
+
+static const uint MAX_TILE_HEIGHT     = 255;                   ///< Maximum allowed tile height
+
+static const uint MIN_MAX_HEIGHTLEVEL = 15;                    ///< Lower bound of maximum allowed heightlevel (in the construction settings)
+static const uint DEF_MAX_HEIGHTLEVEL = 30;                    ///< Default maximum allowed heightlevel (in the construction settings)
+static const uint MAX_MAX_HEIGHTLEVEL = MAX_TILE_HEIGHT;       ///< Upper bound of maximum allowed heightlevel (in the construction settings)
 
 static const uint MIN_SNOWLINE_HEIGHT = 2;                     ///< Minimum snowline height
-static const uint DEF_SNOWLINE_HEIGHT = 7;                     ///< Default snowline height
+static const uint DEF_SNOWLINE_HEIGHT = 15;                    ///< Default snowline height
 static const uint MAX_SNOWLINE_HEIGHT = (MAX_TILE_HEIGHT - 2); ///< Maximum allowed snowline height
 
 
@@ -53,7 +60,7 @@ enum TileType {
  *
  * In randomly generated maps:
  *  TROPICZONE_DESERT: Generated everywhere, if there is neither water nor mountains (TileHeight >= 4) in a certain distance from the tile.
- *  TROPICZONE_RAINFOREST: Genereated everywhere, if there is no desert in a certain distance from the tile.
+ *  TROPICZONE_RAINFOREST: Generated everywhere, if there is no desert in a certain distance from the tile.
  *  TROPICZONE_NORMAL: Everywhere else, i.e. between desert and rainforest and on sea (if you clear the water).
  *
  * In scenarios:
