@@ -1,4 +1,4 @@
-/* $Id: wnd_quartz.mm 23446 2011-12-08 18:13:29Z rubidium $ */
+/* $Id: wnd_quartz.mm 27675 2016-10-31 19:29:01Z michi_cc $ */
 
 /*
  * This file is part of OpenTTD.
@@ -34,6 +34,13 @@
 #include "../../core/math_func.hpp"
 #include "../../gfx_func.h"
 
+/* On some old versions of MAC OS this may not be defined.
+ * Those versions generally only produce code for PPC. So it should be safe to
+ * set this to 0. */
+#ifndef kCGBitmapByteOrder32Host
+#define kCGBitmapByteOrder32Host 0
+#endif
+
 /**
  * Important notice regarding all modifications!!!!!!!
  * There are certain limitations because the file is objective C++.
@@ -50,7 +57,7 @@ class WindowQuartzSubdriver;
 - (void)drawRect:(NSRect)invalidRect;
 @end
 
-class WindowQuartzSubdriver: public CocoaSubdriver {
+class WindowQuartzSubdriver : public CocoaSubdriver {
 private:
 	/**
 	 * This function copies 8bpp pixels from the screen buffer in 32bpp windowed mode.
@@ -103,15 +110,23 @@ static CGColorSpaceRef QZ_GetCorrectColorSpace()
 	static CGColorSpaceRef colorSpace = NULL;
 
 	if (colorSpace == NULL) {
-		CMProfileRef sysProfile;
-
-		if (CMGetSystemProfile(&sysProfile) == noErr) {
-			colorSpace = CGColorSpaceCreateWithPlatformColorSpace(sysProfile);
-			CMCloseProfile(sysProfile);
-		} else {
-			colorSpace = CGColorSpaceCreateDeviceRGB();
+#if (MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_5)
+		if (MacOSVersionIsAtLeast(10, 5, 0)) {
+			colorSpace = CGDisplayCopyColorSpace(CGMainDisplayID());
+		} else
+#endif
+		{
+#if (MAC_OS_X_VERSION_MIN_REQUIRED < MAC_OS_X_VERSION_10_5) && !defined(HAVE_OSX_1011_SDK)
+			CMProfileRef sysProfile;
+			if (CMGetSystemProfile(&sysProfile) == noErr) {
+				colorSpace = CGColorSpaceCreateWithPlatformColorSpace(sysProfile);
+				CMCloseProfile(sysProfile);
+			}
+#endif
 		}
 
+		if (colorSpace == NULL) colorSpace = CGColorSpaceCreateDeviceRGB();
+
 		if (colorSpace == NULL) error("Could not get system colour space. You might need to recalibrate your monitor.");
 	}
 
@@ -144,7 +159,7 @@ static CGColorSpaceRef QZ_GetCorrectColorSpace()
 	/* Calculate total area we are blitting */
 	uint32 blitArea = 0;
 	for (int n = 0; n < dirtyRectCount; n++) {
-		blitArea += dirtyRects[n].size.width * dirtyRects[n].size.height;
+		blitArea += (uint32)(dirtyRects[n].size.width * dirtyRects[n].size.height);
 	}
 
 	/*
@@ -240,12 +255,12 @@ void WindowQuartzSubdriver::GetDeviceInfo()
  */
 bool WindowQuartzSubdriver::ToggleFullscreen()
 {
-#if (MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_7)
-	[this->window toggleFullScreen:this->window];
-	return true;
-#else
+	if ([ this->window respondsToSelector:@selector(toggleFullScreen:) ]) {
+		[ this->window performSelector:@selector(toggleFullScreen:) withObject:this->window ];
+		return true;
+	}
+
 	return false;
-#endif
 }
 
 bool WindowQuartzSubdriver::SetVideoMode(int width, int height, int bpp)
@@ -280,26 +295,27 @@ bool WindowQuartzSubdriver::SetVideoMode(int width, int height, int bpp)
 			return false;
 		}
 
+#if MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_5
 		/* Add built in full-screen support when available (OS X 10.7 and higher)
 		 * This code actually compiles for 10.5 and later, but only makes sense in conjunction
 		 * with the quartz fullscreen support as found only in 10.7 and later
 		 */
-#if (MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_7)
 		if ([this->window respondsToSelector:@selector(toggleFullScreen:)]) {
-			/* Constants needed to build on pre-10.7 systems. Source: NSWindow documentation. */
+#if MAC_OS_X_VERSION_MAX_ALLOWED < MAC_OS_X_VERSION_10_7
+			/* Constants needed to build on pre-10.7 SDKs. Source: NSWindow documentation. */
 			const int NSWindowCollectionBehaviorFullScreenPrimary = 1 << 7;
 			const int NSWindowFullScreenButton = 7;
+#endif
 
-			NSWindowCollectionBehavior behavior = [this->window collectionBehavior];
+			NSWindowCollectionBehavior behavior = [ this->window collectionBehavior ];
 			behavior |= NSWindowCollectionBehaviorFullScreenPrimary;
-			[window setCollectionBehavior:behavior];
+			[ this->window setCollectionBehavior:behavior ];
 
-			NSButton* fullscreenButton =
-			[this->window standardWindowButton:NSWindowFullScreenButton];
-			[fullscreenButton setAction:@selector(toggleFullScreen:)];
-			[fullscreenButton setTarget:this->window];
+			NSButton* fullscreenButton = [ this->window standardWindowButton:NSWindowFullScreenButton ];
+			[ fullscreenButton setAction:@selector(toggleFullScreen:) ];
+			[ fullscreenButton setTarget:this->window ];
 
-			[this->window setCollectionBehavior: NSWindowCollectionBehaviorFullScreenPrimary];
+			[ this->window setCollectionBehavior: NSWindowCollectionBehaviorFullScreenPrimary ];
 		}
 #endif
 
@@ -327,10 +343,10 @@ bool WindowQuartzSubdriver::SetVideoMode(int width, int height, int bpp)
 		[ this->window setContentSize:contentRect.size ];
 
 		/* Ensure frame height - title bar height >= view height */
-		contentRect.size.height = Clamp(height, 0, [ this->window frame ].size.height - 22 /* 22 is the height of title bar of window*/);
+		contentRect.size.height = Clamp(height, 0, (int)[ this->window frame ].size.height - 22 /* 22 is the height of title bar of window*/);
 
 		if (this->cocoaview != nil) {
-			height = contentRect.size.height;
+			height = (int)contentRect.size.height;
 			[ this->cocoaview setFrameSize:contentRect.size ];
 		}
 	}
@@ -513,8 +529,21 @@ CGPoint WindowQuartzSubdriver::PrivateLocalToCG(NSPoint *p)
 
 NSPoint WindowQuartzSubdriver::GetMouseLocation(NSEvent *event)
 {
-	NSPoint pt = [ event locationInWindow ];
-	pt = [ this->cocoaview convertPoint:pt fromView:nil ];
+	NSPoint pt;
+
+	if ( [ event window ] == nil) {
+#if MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_7
+		if ([ this->cocoaview respondsToSelector:@selector(convertRectFromScreen:) ]) {
+			pt = [ this->cocoaview convertPoint:[ [ this->cocoaview window ] convertRectFromScreen:NSMakeRect([ event locationInWindow ].x, [ event locationInWindow ].y, 0, 0) ].origin fromView:nil ];
+		}
+		else
+#endif
+		{
+			pt = [ this->cocoaview convertPoint:[ [ this->cocoaview window ] convertScreenToBase:[ event locationInWindow ] ] fromView:nil ];
+		}
+	} else {
+		pt = [ event locationInWindow ];
+	}
 
 	pt.y = this->window_height - pt.y;
 
@@ -548,8 +577,8 @@ bool WindowQuartzSubdriver::WindowResized()
 
 	NSRect newframe = [ this->cocoaview frame ];
 
-	this->window_width = newframe.size.width;
-	this->window_height = newframe.size.height;
+	this->window_width = (int)newframe.size.width;
+	this->window_height = (int)newframe.size.height;
 
 	/* Create Core Graphics Context */
 	free(this->window_buffer);
